##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

'''Patches App.Product so that a manage_refresh tab is available for
all products that are loaded from the filesystem.
$Id: RefreshPatch.py,v 1.2 2001/01/29 20:59:41 shane Exp $
'''

import InitProduct
import App.Product
import Globals
from Globals import HTMLFile
import Products
import os.path
import sys
import time
from ZODB.Connection import Connection

# Don't let this module be reloaded because it is a patch.
__reload_module__ = 0

origInitializeProduct = App.Product.initializeProduct

orig_setDB = Connection._setDB
global_classes_timestamp = 0


def reloadModule(m):
    reload(m)


def performPatch():

    def initializeProduct(productp, name, home, app):
        # Replacement for App.Product.initializeProduct().
        # Adds the Refresh tab to the product management screens.
        product = origInitializeProduct(productp, name, home, app)
        if product.__dict__.has_key('manage_options'):
            found = 0

            # Remove refresh tabs.
            # Uncomment and restart Zope if RefreshPatch causes
            # unexpected problems.
            #newOptions = []
            #for option in product.manage_options:
            #    if not option.has_key('label') or \
            #       option['label'] != 'Refresh':
            #        newOptions.append(option)
            #product.manage_options = tuple(newOptions)
            #found = 1

            # Ensure this product has a refresh tab.
            for option in product.manage_options:
                if option.has_key('label') and \
                   option['label'] == 'Refresh':
                    # Refresh tab already present.
                    found = 1
                    break
            if not found:
                product.manage_options = product.manage_options + (
                    {'label':'Refresh', 'action':'manage_refresh'},
                    )
        return product

    App.Product.initializeProduct = initializeProduct

    App.Product.Product.manage_refresh = HTMLFile('refresh', globals())

    def getRefreshText(self):
        '''Returns the contents of refresh.txt or None.
        '''
        for productDir in Products.__path__:
            path = os.path.join(productDir, self.id, 'refresh.txt')
            try:
                file = open(path)
                text = file.read()
                file.close()
                return text
            except:
                # Not found here.
                pass
        return None

    App.Product.Product.getRefreshText = getRefreshText

    def _setDB(self, odb):
        '''
        This hook clears the cache for each connection that needs
        to see the updated class definitions.
        '''
        if getattr(self, '_classes_timestamp', 0) != global_classes_timestamp:
            self._classes_timestamp = global_classes_timestamp
            self._invalidated.clear()
            # Clear the cache by re-creating it.
            origCache = self._cache
            from ZODB.cPickleCache import PickleCache
            self._cache = PickleCache(self, origCache.cache_size,
                                      origCache.cache_age)
        rval = orig_setDB(self, odb)
        return rval

    Connection._setDB = _setDB

    def manage_getRefreshableModules(self):
        prefix = "Products.%s" % self.id
        rval = []
        for name, module in sys.modules.items():
            if module and (name == prefix or \
                           name[0:len(prefix) + 1] == prefix + '.'):
                reload_var = getattr(module, '__reload_module__', 1)
                do_reload = 0
                if callable(reload_var) or reload_var:
                    rval.append(name)
        return rval
                
    App.Product.Product.manage_getRefreshableModules = (
        manage_getRefreshableModules)

    def manage_performRefresh(self):
        '''Attempts to perform a refresh operation.  self is the Product
        object.
        '''
        class refreshStatsClass: pass
        refreshStats = refreshStatsClass()
        refreshStats.name = self.id

        reloadList = self.manage_getRefreshableModules()
        reload_funcs = []
        modules = []
        for name in reloadList:
            module = sys.modules[name]
            # Remove the __import_error__ attribute.
            try: del module.__import_error__
            except: pass
            # Try to call __reload_module__ if it exists
            # in the module.
            reload_var = getattr(module, '__reload_module__', 1)
            if callable(reload_var):
                reload_funcs.append(reload_var)
            else:
                modules.append(module)
        try:
            for m in modules:
                reloadModule(m)
            for func in reload_funcs:
                func()

            # Reinitialize the product.
            app = self.getPhysicalRoot()
            InitProduct.import_product(self.id)
            InitProduct.install_product(app, self.id)

            # Update the classes timestamp.
            global global_classes_timestamp
            global_classes_timestamp = time.time()

            refreshStats.reloadList = reloadList
            return refreshStats

        except:
            e = sys.exc_info()
            if hasattr(e[0], '__name__'):
                refreshStats.error_type = e[0].__name__
            else:
                refreshStats.error_type = str(e[0])
            refreshStats.error_value = str(e[1])
            import traceback
            try:
                from cStringIO import StringIO
            except:
                from StringIO import StringIO
            f=StringIO()
            traceback.print_tb(e[2], 200, f)
            refreshStats.error_tb = f.getvalue()
            e = None
            return refreshStats


    App.Product.Product.manage_performRefresh = manage_performRefresh
    
    # Reinitialize permissions for App.Product.Product to
    # get access to new properties.
    Globals.default__class_init__(App.Product.Product)
    
performPatch()

